clear
clc
% Definitions and Parameters
bench = 'Benchmarks.xlsx';
benchsel = 1;
FM =  xlsread(bench,benchsel);
FM = FM';
c = length(FM(1,:));             % Column Size
r = length(FM(:,1));             % Row Size
std = 20;                        % Standard deviation of delay value
mu = 100;                        % Expected mean delay value
VM = zeros(r,c);
FF2 = cell(c,1);
dRate = 0.05;                     % Defect Rate
%% Loop for Generating VM
DM = zeros(r,c);
for i = 1 : c       
    for  j = 1 : r
        VM(j,i) = normrnd(mu,std);      % Generating VM
    end
end
FMh2 = FM;
perf_RAND = max(sum(VM.*FMh2));          % Performance of random mapping
for i = 1 : c       
    for  j = 1 : r
        if rand(1,1) < dRate
            VM(j,i) = 10000;
            DM(j,i) = 1;                % Generating Defects
        end
    end
end

%% Initial Greedy Column Ordering
tic
for y = 1 : length(VM(1,:))
    VMLH(:,y) = sort(VM(:,y));                    % Low to high ordered VM
end
for i = 1 : length(FMh2(1,:))
    for j = 1 : length(VM(1,:))
    PMins(i,j) = sum(VMLH(1:sum(FMh2(:,i)),j));     % Minimum possible value of FMh2 column 'i' on VM column 'j'
    end
end
PMXn = PMins;
PMXn(:,end+1) = 1 : length(PMins(:,1));
PMXn = flipud(sortrows(PMXn));
Mapng = 1:length(FMh2(1,:));
for i = 1 : length(FMh2(1,:))                       % Loop to find column matches
    [mx, pos] = min(PMXn(i,1:end-1));
    Mapng(2,PMXn(i,end)) = pos;
    PMXn(:,pos) = mx*1000;
end
mymap = sortrows(fliplr(Mapng'))';                % New column ordering
FMh2c = FMh2;
for i = 1 : length(FMh2(1,:))                     
    FMh2c(:,i) = FMh2(:,mymap(2,i));                  % New FMh2 construction as FMh2c
end
time_SORT = toc;
%% Hill Climbing with Column Reordering
FMh2 = FMh2c;
FMh2ean = mean(sum(FMh2));                          % Mean transistor count
myreds = []; changecount = 0; mycount = 0;
tic                                             % Timer starts
SS = round(unique(sort(sum(FMh2c))));
Lows = SS(end);
for y = 1 : c
    RR(:,y) = sort(VM(:,y));
    L(y) = sum(RR(1:Lows,y));
end
lb2 = min(L);                                    % Lower bound (lb)
while changecount < c                         % loop for 'n' times column reordering
    for i = 1 : 500                             % Each optimization trial
        mycount = mycount+1;
        t1 = []; t0 = [];
        S = sum(FMh2.*VM);
        [wcase, pos] = max(S);
        p1(mycount) = wcase;                     % Current worst-case delay
        p3(i) = pos;                             % Current position of worst-case delay
        myM(:,1) = FMh2(:,pos);
        myM(:,2) = VM(:,pos);
        for j = 1 : r                           % '1' and '0' row positions on worst-case column
            if myM(j,1) == 1
                t1(:,end+1) = [myM(j,2) j];     % Positions of '1's
            else
                t0(:,end+1) = [myM(j,2) j];     % Positions of '0's
            end
        end
        t0 = rot90(sortrows(t0')); t1 = rot90(sortrows(t1',1)',2);
        for m = 1 : length(t1(1,:))             % Loop for switching '1's and '0's
            for n = 1 : length(t0(1,:))
                m1 = t1(1,m); m0 = t0(1,n);
                FMh2x = FMh2;                       % Temporary FMh2 array as FMh2x
                FMh2x(m1,:) = FMh2(m0,:);
                FMh2x(m0,:) = FMh2(m1,:);
                T = sum(FMh2x.*VM);
                [wcasen, posn] = max(T(:));      % Worst-case and position values after switch
                if wcasen < wcase                % If new worst-case is better than old worst-case break for loops
                    FMh2 = FMh2x;                    % New FMh2 becomes current FMh2x
                    p2(mycount) = wcasen;        % Current worst-case is stored in a matrix
                    break
                end
            end
        end
        if i > 3                                 % If there is no improvement for 3 cases, column reordering occurs
            if p1(mycount) == p1(mycount-2)
                changecount = changecount+1;
                colaccs = zeros(1,c); myaccs = zeros(1,c);
                for k = 1 : c
                    for l = 1 : length(p3(1:end-5))
                        if k == p3(l)
                            colaccs(k) = colaccs(k)+1; % Finding repetitions of worst-case columns
                        end
                    end
                end
                FF2{changecount,1} = FMh2;
                TT2(changecount,1) = max(sum(FMh2.*VM));
                [impacc, impcol] =  max(colaccs);
                myreds(changecount) = impcol;          % Most repeated worst-case column
                myaccs(1,:) = sum(FMh2);
                myaccs(2,:) = 1:c;
                myaccs = sortrows(myaccs');                             % FMh2 columns in low to high order
                newcol = myaccs(mod(changecount,round(c/2))+1,2);     % Column to be switched with worst-case column
                temp1 = FMh2(:,impcol);
                temp2 = FMh2(:,newcol);
                FMh2(:,impcol) = temp2;
                FMh2(:,newcol) = temp1;                                   % Switching columns
                break
            end
        end  
    end
end
time_Climb = toc                               % Runtime output
[perfClimb, perfPOS2] = min(TT2);                           % Lowest delay value achieved
FMF2 = FF2{perfPOS2};
if sum(sum(FMF2.*DM)) == 0
    DEF = 0;
    devRateHClimb = (perf_RAND-perfClimb)/perf_RAND               % Deviation rate from lower bound (lb)
else
    DEF = 1;
    devRateHClimb = []
end

DEF



% figure
% plot(p1)
