clear
clc
%% Definitions and Parameters
bench = 'Benchmarks.xlsx';
benchsel = 5;
FM =  xlsread(bench,benchsel);
FM = FM';
c = length(FM(1,:));             % Column Size
r = length(FM(:,1));             % Row Size
std = 20;                        % Standard deviation of delay value
mu = 100;                        % Expected mean delay value
VM = zeros(r,c);
%% Loop for Generating VM
for i = 1 : c       
    for  j = 1 : r
        VM(j,i) = normrnd(mu,std);      % Generating VM
    end
end
perf_RAND = max(sum(VM.*FM));          % Performance of random mapping
%% Simulated Annealing
counter = 0;
STOP = 0;
SS = unique(sort(sum(FM)));
Lows = SS(end);
Highs = SS(end);
for i = 1 : c
    RR(:,i) = sort(VM(:,i));
    L(i) = sum(RR(1:Lows,i));
    H(i) = sum(RR(end-Highs:end,i));
end
ub = max(H);
lb = min(L);                            % Lower bound
TEMP = ub-lb;                           % Unique temperature value for current FM and VM
exponen = 0.9999;                         % Decay rate of temperature value
R = 1:r; C = 1:c; R0 = 1:r; C0 = 1:c;
R = R'; R0 = R0';
Pold = max(sum(FM.*VM));                % Initial worst-case delay
tic
while STOP == 0                         % Annealing loop
    sel = rand(1,1);                    % Selecting move type (switch row, column or both)
    if sel > 0.67
        mode = 0;
    elseif sel < 0.33
        mode = 1;
    else
        mode = 2;
    end
    if mode == 2;                       % Swithing Rows
        counter = counter +1;
        Rn = R;
        rand1 = round((r-1)*rand(1)+1); % Selecting 2 random rows
        rand2 = round((r-1)*rand(1)+1);
        Rn(rand1) = R(rand2);
        Rn(rand2) = R(rand1);           % Changing row orders on 'R' array
        FMx = MAP(FM, Rn, C0 , r, c);   % Mapping new row order on FM
        Pcur = fitnessM(FMx,VM);        % Finding worst-case delay of new mapping
        pDelt = Pcur - Pold;            % Worst-case difference between old and new mappings
        if rand(1) < exp(-pDelt/TEMP)   % Accepting or declining new mapping regarding to pDelt
            R = Rn;
            Pold = Pcur;
            FM = FMx;
            if Pcur <= lb               % STOP if lower bound is reached
                STOP = 1;
            end
        end
        TEMP = TEMP*exponen;            % New temperature value
        if TEMP < 1
            STOP = 1;                   % STOP if temperature is lower than 1
        end
        PP(counter) = Pold;             % Hold performance value
    end
    
    
    if mode == 1;                       % Switching two columns
        counter = counter +1;
        Cn = C;
        rand1 = round((c-1)*rand(1)+1); % Selecting 2 random columns
        rand2 = round((c-1)*rand(1)+1);
        Cn(rand1) = C(rand2);
        Cn(rand2) = C(rand1);           % Changing column orders on 'C' array
        FMx=MAP(FM, R0, Cn , r, c);     % Mapping new row order on FM
        Pcur = fitnessM(FMx,VM);        % Finding worst-case delay of new mapping
        pDelt = Pcur - Pold;            % Worst-case difference between old and new mappings
        if rand(1) < exp(-pDelt/TEMP)   % Accepting or declining new mapping regarding to pDelt
            C = Cn;
            Pold = Pcur;
            FM = FMx;
            if Pcur < lb                % STOP if lower bound is reached
                STOP = 1;
            end
        end
        TEMP = TEMP*exponen;            % New temperature value
        if TEMP < 1
            STOP = 1;                   % STOP if temperature is lower than 1
        end
        PP(counter) = Pold;             % Hold performance value
    end
    
    
    if mode == 0;                       % Switching both two rows and two columns
        counter = counter +1;
        Cn = C;
        Rn = R;
        rand1 = round((c-1)*rand(1)+1); % Selecting 2 random columns
        rand2 = round((c-1)*rand(1)+1);
        rand3 = round((r-1)*rand(1)+1); % Selecting 2 random rows
        rand4 = round((r-1)*rand(1)+1);
        Cn(rand1) = C(rand2);           
        Cn(rand2) = C(rand1);           % Changing column orders on 'C' array
        Rn(rand3) = R(rand4);           
        Rn(rand4) = R(rand3);           % Changing row orders on 'R' array
        FMx=MAP(FM, Rn, Cn , r, c);     % Mapping new row order on FM
        Pcur = fitnessM(FMx,VM);        % Finding worst-case delay of new mapping
        pDelt = Pcur - Pold;            % Worst-case difference between old and new mappings
        if rand(1) < exp(-pDelt/TEMP)   % Accepting or declining new mapping regarding to pDelt
            R = Rn;
            C = Cn;
            Pold = Pcur;
            FM = FMx;
            if Pcur < lb                % STOP if lower bound is reached
                STOP = 1;
            end
        end
        TEMP = TEMP*exponen;            % New temperature value
        if TEMP < 1
            STOP = 1;                   % STOP if temperature is lower than 1
        end
        PP(counter) = Pold;             % Hold performance value
    end
    
end
time_SA = toc                          % runtime output
% Lim0
SAout = min(PP);                        % Minimum reached worst-case delay as Simulated Annealin Output
devRateSimAnn = (SAout-lb)/lb           % Deviation rate from lower bound