clear
clc
%% Definitions and Parameters
c = 24;             % Column Size
r = 24;             % Row Size
CR = 40;            % Crosspoint Rate
std = 20;           % Standard deviation of delay value
mu = 100;           % Expected mean delay value
VM = zeros(r,c); 
FM = zeros(r,c);
%% Loop for Generating FM and VM
for i = 1 : c       
    count1 = 0; count2 = 0; 
    for  j = 1 : r
        VM(j,i) = normrnd(mu,std);      % Generating VM
        rnd = (100).*rand(1,1);
        if rnd <= CR                    % Generating FM %#ok<BDSCI>
            FM(j,i) = 1;
        else
            FM(j,i) = 0;
        end
    end
    if sum(FM(:,i)) == r                % Ensuring that there is no full '0' column on FM
        rnd = (r).*rand(1,1) + 0.5;
        FM(round(rnd),i) = 0;
    elseif sum(FM(:,i)) == 0
        rnd = (r).*rand(1,1) + 0.5;
        FM(round(rnd),i) = 1;
    end
end
perf_RAND = max(sum(VM.*FM));          % Performance of random mapping
%% Upper / Lower Bounds and Goal Calculations
tic                                     % Timer starts
SS = round(unique(sort(sum(FM))));      
Bot = SS(1);                            % Column with the lowest '1'
Top = SS(end);                          % Column with the highest '1'
ub = Top*mu + 3*sqrt(Top*(std^2));      % Upper bound (worst-case delay for initial (no replication) state)
lb = Top*mu - 3*sqrt(Top*(std^2));      % Lower bound (best-case delay for initial( no replication) state
mid = Top*mu;                           % Expected performance if there is no variations
mybound = Top*mu - 3*sqrt(Top*(std^2)); % Custom Defined Goal (equal to lb, as Goal 2)
%% Column Replications
for i = 1 : c
    myT = sum(FM(:,i));                                 % '1' count for selected column 'i'
    myVar = std;                                        % Standard deviation parameter for column 'i'
    myMean = mu;                                        % Mean parameter for column 'i'
    myworst = myT*myMean + 3*sqrt(myT*(myVar^2));       % Expected worst-case delay for column 'i'
    mycount(i) = 1;
    
    while myworst >= mybound                            % While worst case of column 'i', add a replicate column
        mycount(i) = mycount(i) + 1; 
        myMean = myMean/mycount(i);                     % New mean after replication for column 'i'
        myVar = sqrt((myVar^2)/(mycount(i)^3));         % New standard deviation parameter for column 'i'
        myworst = myT*myMean + 3*sqrt(myT*(myVar^2));   % New worst-case delay
    end
    curworst(i) = myworst;
end
%% Area and Power Overhead Calculations
defpow = 0; reppow = 0;
for k = 1 : c
    defpow = defpow + 1/sum(FM(:,k));               % Initial worst-case power
    reppow = reppow + 1/(sum(FM(:,k))/mycount(k));  % Worst-case power after replication
end
powerovers = reppow/defpow - 1                      % Power overhead rate
delayout = max(curworst);                           % Final worst-case delay
areaovers = sum(mycount)/c - 1                      % Area overhead rate
pdout = reppow*delayout;                            % Power - Delay Product

g1pos = 1 - (mid/ub);                               % Goal 1 position related to upper bound
g2pos = 1 - (lb/ub);                                % Goal 2 position related to upper bound


