clear VM FM VMLH mymap RR myM FMc Mapng PMXn PMins
% clc
%% Definitions and Parameters
c = 48;             % Column Size
r = 48;             % Row Size
CR = 40;            % Crosspoint Rate
std = 30;           % Standard deviation of delay value
mu = 100;           % Expected mean delay value
VM = zeros(r,c); 
FM = zeros(r,c);
%% Loop for Generating FM and VM
for i = 1 : c       
    count1 = 0; count2 = 0; 
    for  j = 1 : r
        VM(j,i) = normrnd(mu,std);      % Generating VM
        rnd = (100).*rand(1,1);
        if rnd <= CR                    % Generating FM %#ok<BDSCI>
            FM(j,i) = 1;
        else
            FM(j,i) = 0;
        end
    end
    if sum(FM(:,i)) == r                % Ensuring that there is no full '0' column on FM
        rnd = (r).*rand(1,1) + 0.5;
        FM(round(rnd),i) = 0;
    elseif sum(FM(:,i)) == 0
        rnd = (r).*rand(1,1) + 0.5;
        FM(round(rnd),i) = 1;
    end
end
FMx = FM;
perf_RAND = max(sum(VM.*FM));          % Performance of random mapping
%% Initial Greedy Column Ordering
tic
for y = 1 : length(VM(1,:))
    VMLH(:,y) = sort(VM(:,y));                    % Low to high ordered VM
end
for i = 1 : length(FM(1,:))
    for j = 1 : length(VM(1,:))
    PMins(i,j) = sum(VMLH(1:sum(FM(:,i)),j));     % Minimum possible value of FM column 'i' on VM column 'j'
    end
end
PMXn = PMins;
PMXn(:,end+1) = 1 : length(PMins(:,1));
PMXn = flipud(sortrows(PMXn));
Mapng = 1:length(FM(1,:));
for i = 1 : length(FM(1,:))                       % Loop to find column matches
    [mx, pos] = min(PMXn(i,1:end-1));
    Mapng(2,PMXn(i,end)) = pos;
    PMXn(:,pos) = mx*1000;
end
mymap = sortrows(fliplr(Mapng'))';                % New column ordering
FMc = FM;
for i = 1 : length(FM(1,:))                     
    FMc(:,i) = FM(:,mymap(2,i));                  % New FM construction as FMc
end
time_SORT = toc;
%% Hill Climbing with Column Reordering
FM = FMc;
Fmean = mean(sum(FM));                          % Mean transistor count
myreds = []; changecount = 0; mycount = 0;
tic                                             % Timer starts
SS = round(unique(sort(sum(FMc))));
Lows = SS(end);
for y = 1 : c
    RR(:,y) = sort(VM(:,y));
    L(y) = sum(RR(1:Lows,y));
end
lb = min(L);                                    % Lower bound (lb)
while changecount < c                       % loop for 'n' times column reordering
    for i = 1 : 500                             % Each optimization trial
        mycount = mycount+1;
        t1 = []; t0 = [];
        S = sum(FM.*VM);
        [wcase, pos] = max(S(:));
        p1(mycount) = wcase;                     % Current worst-case delay
        p3(i) = pos;                             % Current position of worst-case delay
        myM(:,1) = FM(:,pos);
        myM(:,2) = VM(:,pos);
        for j = 1 : r                           % '1' and '0' row positions on worst-case column
            if myM(j,1) == 1
                t1(:,end+1) = [myM(j,2) j];
            else
                t0(:,end+1) = [myM(j,2) j];
            end
        end
        t0 = rot90(sortrows(t0')); t1 = rot90(sortrows(t1',1)',2);
        for m = 1 : length(t1(1,:))             % Loop for switching '1's and '0's
            for n = 1 : length(t0(1,:))
                m1 = t1(1,m); m0 = t0(1,n);
                FMx = FM;                       % Temporary FM array as FMx
                FMx(m1,:) = FM(m0,:);
                FMx(m0,:) = FM(m1,:);
                T = sum(FMx.*VM);
                [wcasen, posn] = max(T(:));      % Worst-case and position values after switch
                if wcasen < wcase                % If new worst-case is better than old worst-case break for loops
                    FM = FMx;                    % New FM becomes current FMx
                    p2(mycount) = wcasen;        % Current worst-case is stored in a matrix
                    break
                end
            end
        end
        if i > 3                                 % If there is no improvement for 3 cases, column reordering occurs
            if p1(mycount) == p1(mycount-2)
                changecount = changecount+1;
                colaccs = zeros(1,c); myaccs = zeros(1,c);
                for k = 1 : c
                    for l = 1 : length(p3(1:end-5))
                        if k == p3(l)
                            colaccs(k) = colaccs(k)+1; % Finding repetitions of worst-case columns
                        end
                    end
                end
                [impacc, impcol] =  max(colaccs);
                myreds(changecount) = impcol;          % Most repeated worst-case column
                myaccs(1,:) = sum(FM);
                myaccs(2,:) = 1:c;
                myaccs = sortrows(myaccs');                             % FM columns in low to high order
                newcol = myaccs(mod(changecount,round(Fmean))+1,2);     % Column to be switched with worst-case column
                temp1 = FM(:,impcol);
                temp2 = FM(:,newcol);
                FM(:,impcol) = temp2;
                FM(:,newcol) = temp1;                                   % Switching columns
                break
            end
        end  
    end
end
time_Climb = toc;                               % Runtime output
perfClimb = min(p1);                            % Lowest delay value achieved
devRateHClimb = (perf_RAND-perfClimb)/perf_RAND;     % Deviation rate from lower bound (lb)
% plot(p1)
