clear
clc
%% Definitions and Parameters
c = 24;             % Column Size
r = 24;             % Row Size
CR = 40;            % Crosspoint Rate
std = 20;           % Standard deviation of delay value
mu = 100;           % Expected mean delay value
VM = zeros(r,c); 
FM = zeros(r,c);
%% Loop for Generating FM and VM
for i = 1 : c       
    count1 = 0; count2 = 0; 
    for  j = 1 : r
        VM(j,i) = normrnd(mu,std);      % Generating VM
        rnd = (100).*rand(1,1);
        if rnd <= CR                    % Generating FM %#ok<BDSCI>
            FM(j,i) = 1;
        else
            FM(j,i) = 0;
        end
    end
    if sum(FM(:,i)) == r                % Ensuring that there is no full '0' column on FM
        rnd = (r).*rand(1,1) + 0.5;
        FM(round(rnd),i) = 0;
    elseif sum(FM(:,i)) == 0
        rnd = (r).*rand(1,1) + 0.5;
        FM(round(rnd),i) = 1;
    end
end
perf_RAND = max(sum(VM.*FM));          % Performance of random mapping
%% Matrix Reducing
FMlb = FM;                                  % Definition of FM for reducing by lb
FMlim = FM;                                 % Definition of FM for reducing by lim
for i = 1 : c                               
    VMSLH(:,i) = sort(VM(:,i));             % Sorting each VM column low to high
    VMSHL(:,i) = flipud(sort(VM(:,i)));     % Sorting each VM column high to low
end
for i = 1 : c                               
    Ti = sum(FM(:,i));                      % '1' count of FM column 'i'
    for j = 1 : c
        ubs(i,j) = sum(VMSHL(1:Ti,j));    % Highest values on each VM column of FM column 'i'
        lbs(i,j) = sum(VMSLH(1:Ti,j));    % Lowest values on each VM column of FM column 'i'
    end
    ub(i) = max(ubs(i,:));                % Upper bound delay of FM column 'i'
    lb(i) = min(lbs(i,:));                % Lower bound delay of FM column 'i'
    lim(i) = max(lbs(i,:));               % Highest lower bound of FM column 'i'
end
lbmax = max(lb);                            % Lower bound 'lb' delay value
limmax = max(lim);                          % Highest lowet bound 'lim' delay value
for i = c : -1 : 1
    if ub(i) < lbmax
        FMlb(:,i) = [];                     % Reducing columns for 'lb' delay value
    end
end
for i = c : -1 : 1
    if ub(i) < limmax
        FMlim(:,i) = [];                    % Reducing columns for 'lim' delay value
    end
end

defsize = size(FM)                          % Initial matrix size
lbsize = size(FMlb)                         % Matrix size after reducing for 'lb'
limsize = size(FMlim)                       % Matrix size after reducing for 'lim'

