clear
clc
%% Definitions and Parameters
c = 24;             % Column Size
r = 24;             % Row Size
CR = 40;            % Crosspoint Rate
std = 20;           % Standard deviation of delay value
mu = 100;           % Expected mean delay value
VM = zeros(r,c); 
FM = zeros(r,c);
%% Loop for Generating FM and VM
for i = 1 : c       
    count1 = 0; count2 = 0; 
    for  j = 1 : r
        VM(j,i) = normrnd(mu,std);      % Generating VM
        rnd = (100).*rand(1,1);
        if rnd <= CR                    % Generating FM %#ok<BDSCI>
            FM(j,i) = 1;
        else
            FM(j,i) = 0;
        end
    end
    if sum(FM(:,i)) == r                % Ensuring that there is no full '0' column on FM
        rnd = (r).*rand(1,1) + 0.5;
        FM(round(rnd),i) = 0;
    elseif sum(FM(:,i)) == 0
        rnd = (r).*rand(1,1) + 0.5;
        FM(round(rnd),i) = 1;
    end
end
perf_RAND = max(sum(VM.*FM));          % Performance of random mapping
%% Lower bound (lb) calculation
SS = unique(sort(sum(FM)));
Lows = SS(end);
for i = 1 : c
    RR(:,i) = sort(VM(:,i));
    L(i) = sum(RR(1:Lows,i));
end
lb = min(L);
%% Random Solution Generation
population = 200;                                                           % Different ordering solutions of rows and columns
tic                                                                     % Timer starts
[ParentRow, ParentCol] = PopGenerate(population,r,c);                       % Generation random permutations
[FitnsM] = TestCurrentGen(ParentRow, ParentCol, FM, r, c, VM);          % Worst-case delays of each (row x column) solution
PROBout = min(nonzeros(FitnsM));                                        % Lowest achieved delay value of all solutions
devRateRandom = (PROBout-lb)/lb                                         % Deviation rate from lower bound
time_RAND = toc                                                         % Runtime output
